--------------------------------------------------------
--  Paolo Santi - 2007                                --
--  Railway (il ritorno) 1.1                          --
--  Universita' di Padova                             --
--  Anno accademico 2006/2007                         --
--  Laurea Magistrale in Informatica                  --
--  Corso di Sistemi Concorrenti e Distribuiti        --
--                                                    --
--  Rielaborazione di un progetto di Alessandro Temil --
--                                                    --
--------------------------------------------------------
--  Copyright (C) 2006                                --
--  Alessandro Temil                                  --
--  Railway 1.0                                       --
--  Concurrent and Distributed Systems class project  --
--  Master Degree in Computer Science                 --
--  Academic year 2005/6                              --
--  Dept. of Pure and Applied Mathematics             --
--  University of Padua, Italy                        --
--                                                    --
--  This program is free software; you can            --
--  redistribute it and/or modify it under the terms  --
--  of the GNU General Public License as published    --
--  by the Free Software Foundation; either           --
--  version 2 of the License, or (at your option)     --
--  any later version.                                --
--------------------------------------------------------

with Railway.Platform;       use Railway.Platform;
with Railway.Network;        use Railway.Network;
with Ada.Exceptions;        use Ada.Exceptions;

with Ada.Text_IO;        use Ada.Text_IO;

package body Railway.Stations is


   --  attribute 'Partition_ID of unit U returns an integer
   --+ which is the unique id that identifies the partition
   --+ on which unit U is located
   Local_Partition_ID  : constant Integer := Railway.Stations'Partition_Id;


   --  Platform Names
   PLNULL   : aliased String := ("NULL-PLATFORM");
   PLVENEZIA1   : aliased String := ("VE-1"
                                     & "{" & Integer'Image (Local_Partition_ID) & "}");
   PLVENEZIA2   : aliased String := ("VE-2"
                                     & "{" & Integer'Image (Local_Partition_ID) & "}");
   PLMADRID1    : aliased String := ("MA-1"
                                     & "{" & Integer'Image (Local_Partition_ID) & "}");
   PLMADRID2    : aliased String := ("MA-2"
                                     & "{" & Integer'Image (Local_Partition_ID) & "}");
   PLPARIS1     : aliased String := ("PA-1"
                                     & "{" & Integer'Image (Local_Partition_ID) & "}");
   PLPARIS2     : aliased String := ("PA-2"
                                     & "{" & Integer'Image (Local_Partition_ID) & "}");
   PLBRUXELLES1 : aliased String := ("BR-1"
                                     & "{" & Integer'Image (Local_Partition_ID) & "}");
   PLBRUXELLES2 : aliased String := ("BR-2"
                                     & "{" & Integer'Image (Local_Partition_ID) & "}");
   PLWIEN1      : aliased String := ("WI-1"
                                     & "{" & Integer'Image (Local_Partition_ID) & "}");
   PLWIEN2      : aliased String := ("WI-2"
                                     & "{" & Integer'Image (Local_Partition_ID) & "}");

   --  Platforms
   Pl_Null : aliased Platform_T (PLNULL'Access,
                                 NULLSTNAME'Access);
   Pl_Venezia_1 : aliased Platform_T (PLVENEZIA1'Access,
                                      VENEZIASTNAME'Access);
   Pl_Venezia_2 : aliased Platform_T (PLVENEZIA2'Access,
                                      VENEZIASTNAME'Access);

   Pl_Madrid_1 : aliased Platform_T (PLMADRID1'Access,
                                     MADRIDSTNAME'Access);
   Pl_Madrid_2 : aliased Platform_T (PLMADRID2'Access,
                                     MADRIDSTNAME'Access);

   Pl_Paris_1 : aliased Platform_T (PLPARIS1'Access,
                                    PARISSTNAME'Access);

   Pl_Paris_2 : aliased Platform_T (PLPARIS2'Access,
                                    PARISSTNAME'Access);

   Pl_Bruxelles_1 : aliased Platform_T (PLBRUXELLES1'Access,
                                        BRUXELLESSTNAME'Access);
   Pl_Bruxelles_2 : aliased Platform_T (PLBRUXELLES2'Access,
                                        BRUXELLESSTNAME'Access);

   Pl_Wien_1 : aliased Platform_T (PLWIEN1'Access,
                                   WIENSTNAME'Access);
   Pl_Wien_2 : aliased Platform_T (PLWIEN2'Access,
                                   WIENSTNAME'Access);


   --  AllPlatforms  : aliased array(Platform_ID_T) of Platform_Ref_T :=
   AllPlatforms  : aliased AllPlatforms_T :=
     (Pl_Null'Access,
      Pl_Venezia_1'Access,
      Pl_Venezia_2'Access,
      Pl_Madrid_1'Access,
      Pl_Madrid_2'Access,
      Pl_Paris_1'Access,
      Pl_Paris_2'Access,
      Pl_Bruxelles_1'Access,
      Pl_Bruxelles_2'Access,
      Pl_Wien_1'Access,
      Pl_Wien_2'Access);

   --  take a minimum traveler's route and expand it
   --  filling all fields required: starting, ending etc...
   --
   --  forward declaration - see below
   procedure Make_Route(Legs : in Travel_Route_T;
                        ExtLegs : out Travel_Ext_Route_Ref_T);


   --  train arrived at the Platform
   procedure Train_Arrived_Platform
     (Platform_Number : Platform_ID_T;
      Train_Name : String;
      Train_Number : Range_Trains;
      Free_Seats : Integer;
      Seats : Integer;
      Destination : String) is
   begin
      AllPlatforms(Platform_Number).Train_Arrived(Train_Name,
                                                  Train_Number,
                                                  Free_Seats,
                                                  Seats,
                                                  Destination);
   end Train_Arrived_Platform;


   --  train enqueues at the Platform
   procedure Stopping_Platform
     (Platform_Number : Platform_ID_T;
      Train_Name : String;
      Train_Number : Range_Trains;
      Free_Seats : Integer;
      Seats : Integer;
      Destination : String) is
   begin
      AllPlatforms(Platform_Number).Stopping(Train_Name,
                                             Train_Number,
                                             Free_Seats,
                                             Seats,
                                             Destination);
   end Stopping_Platform;


   --  train boards Passengers
   procedure Boarding_Platform
     (Platform_Number : Platform_ID_T;
      Train_Name : String;
      Next_Stage : Integer;
      Destination : String) is
   begin
      AllPlatforms(Platform_Number).Boarding(Train_Name,
                                             Next_Stage,
                                             Destination);
      null;
   end Boarding_Platform;


   --  train Departs
   procedure Close_Doors_Platform
     (Platform_Number : Platform_ID_T;
      Train_Name : String;
      Free_Seats : out Integer;
      Destination : String) is
   begin
      AllPlatforms(Platform_Number).Close_Doors(Train_Name,
                                                Free_Seats,
                                                Destination);
   end Close_Doors_Platform;

   --  utility function get name of platform giving its ID
   function Get_Platform_Name (Platform_Number : Platform_ID_T) return String is
   begin
      return AllPlatforms(Platform_Number).Name.all;
   end Get_Platform_Name;


   --  utility function get ID of station giving platform ID
   function Get_Platform_Station (Platform_Number : Platform_ID_T) return String is
   begin
      return AllPlatforms(Platform_Number).Station.all;
   end Get_Platform_Station;

   --  passenger make a travel
   procedure Passenger_Travel
     (Passenger_Route : Travel_Route_T;
      Travel_Index : in out Integer;
      Passenger_Name : String) is
      Ticket_Station : Station_T;
      Passenger_Ext_Route : Travel_Ext_Route_Ref_T;
   begin
      --        Put_Line ("Passenger_Travel = " & Passenger_Name);
      Make_Route(Passenger_Route, Passenger_Ext_Route);
      Ticket_Station := Platforms_Station_Map
        (Passenger_Ext_Route(Passenger_Ext_Route'First).Plat_From);
      All_Ticket_Counters(Ticket_Station).
        Buy_Ticket(Passenger_Ext_Route.all, Travel_Index, Passenger_Name);
   end Passenger_Travel;


   --  local utility function
   --
   --  take a minimum traveler's route and expand it
   --  filling all fields required: starting, ending etc...
   procedure Make_Route(Legs : in Travel_Route_T;
                        ExtLegs : out Travel_Ext_Route_Ref_T) is

      First_Segment   : Integer;       --  starting platform of leg
      Last_Segment    : Integer;       --  ending platform of leg
      Temp_segment    : Integer;
      Mem_segment     : Integer;
      Plat_First      : Platform_ID_T; --  starting platform of leg
      Plat_Last       : Platform_ID_T; --  ending platform of leg
      Plat_Temp       : Platform_ID_T;
      Plat_Mem        : Platform_ID_T;
      Invalid_Index   : Integer;

   begin
      ExtLegs := new Travel_Ext_Route_T(Legs'First .. Legs'Last);

      --  explicit start & end stage and platform for each train requested
      for I in Legs'Range loop
         --  train identifier (given)
         ExtLegs(I).Train := Legs(I).Train;
         --  starting station of leg (given)
         ExtLegs(I).Station_From  := Legs(I).Station_From;
         --  ending station of leg (given)
         ExtLegs(I).Station_To  := Legs(I).Station_To;

         Invalid_Index := All_Trains_Def(ExtLegs(I).Train).Route'Last + 1;
         First_Segment := Invalid_Index;
         Last_Segment := Invalid_Index;

         --  search the first stage of trip...
         for J in  All_Trains_Def(ExtLegs(I).Train).Route'Range loop
            Plat_Temp := All_Trains_Def(ExtLegs (I).Train).Route(J).
              Departure_Platform;

            --              Put_Line ("First Plat_Temp_Station = " & Get_Platform_Station(Plat_Temp));
            if Platforms_Station_Map(Plat_Temp)
              = ExtLegs(I).Station_From then
               First_Segment := J;
               Plat_First := Plat_Temp;
               exit;
            end if;
         end loop;

         if First_Segment /= Invalid_Index then
            --  search last stage of trip
            Temp_segment := First_Segment+1;
            Mem_segment := First_Segment;
            Plat_Mem := Plat_First;
            for J in All_Trains_Def(ExtLegs(I).Train).Route'First ..
              All_Trains_Def(ExtLegs(I).Train).Route'Last-1 loop

               --  if end reached wrap to beginning of route
               if Temp_segment > All_Trains_Def(ExtLegs(I).Train).Route'Last then
                  Temp_segment := All_Trains_Def(ExtLegs(I).Train).Route'First;
               end if;

               Plat_Temp := All_Trains_Def(ExtLegs (I).Train).Route(Temp_segment).
                 Departure_Platform;
               --  the first found could not be the nearest to end
               --  take another as start stage

               --                 Put_Line ("Next Plat_Temp_Station = " & Get_Platform_Station(Plat_Temp));

               if Platforms_Station_Map(Plat_Temp)
                 = ExtLegs(I).Station_From then
                  First_Segment := Temp_segment;
                  Plat_First := Plat_Temp;
               end if;
               if Platforms_Station_Map(Plat_Temp)
                 = ExtLegs (I).Station_To then
                  --                    Last_Segment := Mem_segment;
                  --                    Plat_Last := Plat_Mem;
                  Last_Segment := Temp_segment;
                  Plat_Last := Plat_Temp;
                  exit;
               end if;
               --  fix the last found
               Plat_Mem := Plat_Temp;
               Mem_segment := Temp_segment;
               --  skip to next segment
               Temp_segment := Temp_segment+1;
            end loop;
         end if;

         if First_Segment = Invalid_Index then
            Raise_Exception
              (Program_Error'Identity,
               "Traveler.Make_Route Fatal error: first segment not found!");
         end if;
         if Last_Segment = First_Segment then
            Raise_Exception
              (Program_Error'Identity,
               "Traveler.Make_Route Fatal error: last segment not found!");
         end if;

         ExtLegs(I).Plat_From := Plat_First;
         ExtLegs(I).Plat_To := Plat_Last;

         ExtLegs(I).Stg_From := First_Segment;
         ExtLegs(I).Stg_To := Last_Segment;
         ExtLegs(I).Stg_Tot := All_Trains_Def(ExtLegs(I).Train).Route'Last;
         ExtLegs(I).Train_Type := All_Trains_Def(ExtLegs(I).Train).category;

         --           Put_Line ("Train = " & Integer'Image(ExtLegs(I).Train));
         --           Put_Line ("First_Segment = " & Integer'Image(First_Segment));
         --           Put_Line ("Last_Segment  = " & Integer'Image(Last_Segment));
         --           Put_Line ("Plat_First    = " & Platforms_Names(Plat_First));
         --           Put_Line ("Plat_Last     = " & Platforms_Names(Plat_Last));
         --           Put_Line ("Plat_First_Station    = " & Get_Platform_Station(Plat_First));
         --           Put_Line ("Plat_Last_Station     = " & Get_Platform_Station(Plat_Last));
      end loop;
   end Make_Route;

begin

   --  makes a reference to all platforms used in railway-platform
   AllPlatformsReference := AllPlatforms'Access;

   --  sets up an array of references to all ticket DBs used in railway-platform
   for t in All_Ticket_DB'Range loop
      All_The_Ticket_DB_Refs(t) := All_Ticket_DB(t);
   end loop;


end Railway.Stations;
