--------------------------------------------------------
--  Paolo Santi - 2007                                --
--  Railway (il ritorno) 1.1                          --
--  Universita' di Padova                             --
--  Anno accademico 2006/2007                         --
--  Laurea Magistrale in Informatica                  --
--  Corso di Sistemi Concorrenti e Distribuiti        --
--                                                    --
--  Rielaborazione di un progetto di Alessandro Temil --
--                                                    --
--------------------------------------------------------
--  Copyright (C) 2006                                --
--  Alessandro Temil                                  --
--  Railway 1.0                                       --
--  Concurrent and Distributed Systems class project  --
--  Master Degree in Computer Science                 --
--  Academic year 2005/6                              --
--  Dept. of Pure and Applied Mathematics             --
--  University of Padua, Italy                        --
--                                                    --
--  This program is free software; you can            --
--  redistribute it and/or modify it under the terms  --
--  of the GNU General Public License as published    --
--  by the Free Software Foundation; either           --
--  version 2 of the License, or (at your option)     --
--  any later version.                                --
--------------------------------------------------------
with Railway.Track;          use Railway.Track;

package body Railway.Tracks is

   --  attribute 'Partition_ID of unit U returns an integer
   --+ which is the unique id that identifies the partition
   --+ on which unit U is located
   Local_Partition_ID  : constant Integer := Railway.Tracks'Partition_Id;

   --  Track Names
   Null_Track : aliased String  := ("Null Track - not found");

   Br_Wi_Track : aliased String := ("Bruxelles - Wien (bidirectional)"
                                   & "{" & Integer'Image (Local_Partition_ID) & "}");

   Br_Pa_Track : aliased String := ("Bruxelles - Paris (unidirectional)"
                                   & "{" & Integer'Image (Local_Partition_ID) & "}");
   Pa_Br_Track : aliased String := ("Paris - Bruxelles (unidirectional)"
                                   & "{" & Integer'Image (Local_Partition_ID) & "}");

   Pa_Ve_Track : aliased String := ("Paris - Venezia (unidirectional)"
                                   & "{" & Integer'Image (Local_Partition_ID) & "}");
   Ve_Pa_Track : aliased String := ("Venezia - Paris (unidirectional)"
                                   & "{" & Integer'Image (Local_Partition_ID) & "}");

   Pa_Ma_Track : aliased String := ("Paris - Madrid (unidirectional)"
                                   & "{" & Integer'Image (Local_Partition_ID) & "}");
   Ma_Pa_Track : aliased String := ("Madrid - Paris (unidirectional)"
                                   & "{" & Integer'Image (Local_Partition_ID) & "}");

   --  Tracks
   --+-------
   --  DISCLAIMER: lengths are assigned to tracks
   --+ with no regard to real physical distances
   --+-------
   NULL_TR : aliased Track_T (1, Null_Track'Access);

   --  Bruxell - Wien (single track)
   BR_WI : aliased Track_T (5_000, Br_Wi_Track'Access);

   --  Bruxell - Paris (double track)
   BR_PA : aliased Track_T (3_000, Br_Pa_Track'Access);
   PA_BR : aliased Track_T (3_000, Pa_Br_Track'Access);

   --  Venice - Paris (double track)
   VE_PA : aliased Track_T (3_000, Ve_Pa_Track'Access);
   PA_VE : aliased Track_T (3_000, Pa_Ve_Track'Access);

   --  Paris - Madrid (double track)
   PA_MA : aliased Track_T (4_000, Pa_Ma_Track'Access);
   MA_PA : aliased Track_T (4_000, Ma_Pa_Track'Access);

   --  Segments
   Null_Segment : constant Segment_T := NULL_TR'Access;

   Bruxelles_Wien : constant Segment_T := BR_WI'Access;
   Wien_Bruxelles : constant Segment_T := BR_WI'Access;

   Paris_Bruxelles : constant Segment_T := PA_BR'Access;
   Bruxelles_Paris : constant Segment_T := BR_PA'Access;

   Venezia_Paris : constant Segment_T := VE_PA'Access;
   Paris_Venezia : constant Segment_T := PA_VE'Access;

   Paris_Madrid : constant Segment_T := PA_MA'Access;
   Madrid_Paris : constant Segment_T := MA_PA'Access;

   AllSegments  : constant array(Track_ID_T) of Segment_T :=
     (Null_Segment,
      Bruxelles_Wien,
      Wien_Bruxelles,
      Paris_Bruxelles,
      Bruxelles_Paris,
      Venezia_Paris,
      Paris_Venezia,
      Paris_Madrid,
      Madrid_Paris);

   procedure Enter_Track (Track_Number : Track_ID_T;
                         Train_Number : Integer;
                         Train_Name : String;
                         Seats : Integer;
                          Seatsmax : Integer) is
      Local_Train_Name : aliased String := Train_Name;
   begin
      AllSegments(Track_Number).Enter(Train_Number,
                                      Local_Train_Name'Access,
                                      Seats,
                                      Seatsmax);
   end Enter_Track;


      --  release the track
      --+ (formal parameter not used explicitly
      --+  but needed for distributed dispatching)
   procedure Leave_Track (Track_Number : Track_ID_T;
                         Train_Name : String) is
      Local_Train_Name : aliased String := Train_Name;
   begin
      AllSegments(Track_Number).Leave(Local_Train_Name'Access);
   end Leave_Track;

   --  gets the length in Km of track with given ID
   function Get_Track_Length(Track_Number : Track_ID_T) return Positive is
   begin
      return AllSegments(Track_Number).Km;
   end Get_Track_Length;

   --  gets the traversing time in seconds of track with given ID
   --  it seems not realistic because it's indedependent from train!
   function Get_Traversing_Time (Track_Number : Track_ID_T) return Integer is
   begin
      return AllSegments(Track_Number).Km * Scaling_Factor;
   end Get_Traversing_Time;

end Railway.Tracks;
