--------------------------------------------------------
--  Paolo Santi - 2007                                --
--  Railway (il ritorno) 1.1                          --
--  Universita' di Padova                             --
--  Anno accademico 2006/2007                         --
--  Laurea Magistrale in Informatica                  --
--  Corso di Sistemi Concorrenti e Distribuiti        --
--                                                    --
--  Rielaborazione di un progetto di Alessandro Temil --
--                                                    --
--------------------------------------------------------
with Gdk;             use Gdk;
with Gtk;             use Gtk;
with Gdk.Color;       use Gdk.Color;
with Gdk.Drawable;    use Gdk.Drawable;
with Gdk.Event;       use Gdk.Event;
with Gdk.Font;
with Gdk.GC;          use Gdk.GC;
with Gtk.Widget;      use Gtk.Widget;
with Gdk.Window;      use Gdk.Window;
with Gtk.Object;      use Gtk.Object;
with Gtk.Handlers;    use Gtk.Handlers;

package body Net_View is



   --  Net Map definitions...
   type StationMapDef_T is record
      X : Integer;
      Y : Integer;
      ShortName : String(1..2);
   end record;

   AllStationsMapDefs : constant array (Station_T) of StationMapDef_T :=
     (
        (X => 150, Y => 100, ShortName => "PA"),
      (X => 250, Y =>  10, ShortName => "BR"),
      (X => 10, Y => 300, ShortName => "MA"),
      (X => 200, Y => 280, ShortName => "VE"),
      (X => 300, Y => 150, ShortName => "WI"),
      (X =>  -1, Y =>  -1, ShortName => "NU")
     );

   type TrackMapDef_T is record
      StatFrom : Station_T;
      StatTo   : Station_T;
   end record;

   AllTracksMapDefs : constant array (Track_ID_T) of TrackMapDef_T :=
     (
      Null_Track_ID => (StatFrom =>  NullStation, StatTo =>  NullStation),
      Bruxelles_Wien_Track_ID  => (StatFrom => Bruxelles, StatTo => Wien),
      Wien_Bruxelles_Track_ID  => (StatFrom => Wien, StatTo => Bruxelles),

      Paris_Bruxelles_Track_ID => (StatFrom => Paris, StatTo => Bruxelles),
      Bruxelles_Paris_Track_ID => (StatFrom => Bruxelles, StatTo => Paris),

      Venezia_Paris_Track_ID   => (StatFrom => Venezia, StatTo => Paris),
      Paris_Venezia_Track_ID   => (StatFrom => Paris, StatTo => Venezia),

      Paris_Madrid_Track_ID    => (StatFrom => Paris, StatTo => Madrid),
      Madrid_Paris_Track_ID    => (StatFrom => Madrid, StatTo => Paris)
     );


   --  Warning: Creating a widget from scratch is a difficult matter.
   --  You should not try that unless you already know quite well how
   --  gtk works. There are a lot of issues to consider for handling
   --  signals, requesting a size, drawing the widget, ...
   --  Please have a look at the tutorial for gtk+ itself to see a
   --  brief summary of all the things to take care of.


   --  This pointer will keep a pointer to the C 'class record' for
   --  gtk. To avoid allocating memory for each widget, this may be done
   --  only once, and reused
   Class_Record : constant GObject_Class := Uninitialized_Class;

   --  Array of the signals created for this widget

   --     Signals : Chars_Ptr_Array := "bullseye" + "missed";


   --     package Internal_Cb is new Handlers.Callback (RailMap_Widget_Record);

   --  The type of callbacks for the signals above. This is used only to
   --  emit the signals.

   --  Note: To create a real-life widget, you would have to do more
   --  things than are done in this simple example:
   --    * The "expose_event" signal actually passes the Area to
   --      redraw, which can speed things up a lot if used correctly
   --    * You should probably use some double-buffers to avoid the
   --      flickering that is visible in this simple example
   --    * Connect a function to the "destroy" callback to take care of
   --      the finalization of the object.

   package Return_Boolean_Cb is new Handlers.Return_Callback
     (RailMap_Widget_Record, Boolean);

   --  Define our own marshaller, since this is not one of the
   --  standard one.
   package Size_Cb is new Handlers.Callback (RailMap_Widget_Record);
   package Requisition_Marshaller is new Size_Cb.Marshallers.Generic_Marshaller
     (Gtk_Requisition_Access, Gtk.Widget.Get_Requisition);

   package Allocation_Cb is new Handlers.Callback (RailMap_Widget_Record);
   package Allocation_Marshaller is new
     Allocation_Cb.Marshallers.Generic_Marshaller
       (Gtk_Allocation_Access, Gtk.Widget.Get_Allocation);

   -----------------
   -- Draw_RailMap --
   -----------------

   function Draw_RailMap
     (Widget  : access RailMap_Widget_Record'Class)
      return Boolean;

   function Draw_RailMap
     (Widget  : access RailMap_Widget_Record'Class)
      return Boolean
   --  This function is called when we need to redraw the widget (for
   --  instance whenever part of it has been cleared
   is
      Width, Height : Gint;
      Win  : constant Gdk.Window.Gdk_Window := Get_Window (Widget);
      Font : Gdk.Font.Gdk_Font;
   begin
      if Widget.Gc_Trains = null then
         declare
            Color : Gdk_Color;
         begin
            Color := Gdk.Color.Parse ("Red");
            Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);

            Gdk.GC.Gdk_New (Widget.Gc_Trains, Win);
            Gdk.GC.Set_Foreground (Widget.Gc_Trains, Color);

            Color := Gdk.Color.Parse ("Black");
            Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);

            Gdk.GC.Gdk_New (Widget.Gc_Line, Win);
            Gdk.GC.Set_Foreground (Widget.Gc_Line, Color);
         end;
      end if;
      Gdk.Drawable.Get_Size (Win, Width, Height);

      for track in AllTracksMapDefs'Range loop
         declare
            from : constant Station_T := AllTracksMapDefs(track).StatFrom;
            to   : constant Station_T := AllTracksMapDefs(track).StatTo;
            x0 : constant Gint := Gint(AllStationsMapDefs(from).X);
            y0 : constant Gint := Gint(AllStationsMapDefs(from).Y);
            x1 : constant Gint := Gint(AllStationsMapDefs(to).X);
            y1 : constant Gint := Gint(AllStationsMapDefs(to).Y);
         begin
            if from/=NullStation and to/=NullStation then
               Gdk.Drawable.Draw_Line
                 (Drawable => Win,
                  Gc => Widget.Gc_Line,
                  X1 => x0,
                  Y1 => y0,
                  X2 => x1,
                  Y2 => y1);
            end if;
         end;
      end loop;

      Gdk.Font.Load (Font,
                     "-*-*-*-o-*-*-10-*-*-*-*-*-*-*");

      for stat in AllStationsMapDefs'Range loop
         Gdk.Drawable.Draw_Text
           (Drawable => Win,
            Font     => Font,
            Gc       => Widget.Gc_Line,
            X        => Gint(AllStationsMapDefs(stat).X + 20),
            Y        => Gint(AllStationsMapDefs(stat).Y),
            Text     => AllStationsMapDefs(stat).ShortName);
      end loop;

      for train in Range_Trains'First .. Widget.MaxTrainUsed loop
         declare
            from : constant Station_T := Widget.AllTrainsPositions(train).stat_from;
            to   : constant Station_T := Widget.AllTrainsPositions(train).stat_to;
            x0 : constant Gint := Gint(AllStationsMapDefs(from).X);
            y0 : constant Gint := Gint(AllStationsMapDefs(from).Y);
            x1 : constant Gint := Gint(AllStationsMapDefs(to).X);
            y1 : constant Gint := Gint(AllStationsMapDefs(to).Y);
            pos  : constant Gint := Gint(Widget.AllTrainsPositions(train).pos);
            over : constant Gint := Gint(Widget.AllTrainsPositions(train).over);
            xd   : constant Gint := (x1-x0)*pos;
            yd   : constant Gint := (y1-y0)*pos;
            Name : constant String :=   --  "T" &
              Integer'Image(train);
         begin
            if from/=NullStation and to/=NullStation
              and over>0 then
               Gdk.Drawable.Draw_Text
                 (Drawable => Win,
                  Font     => Font,
                  Gc       => Widget.Gc_Trains,
                  X        => xd/over + x0,
                  Y        => yd/over + y0,
                  Text     => Name);
            end if;
         end;
      end loop;

      return True;
   end Draw_RailMap;

   ------------------
   -- Size_Request --
   ------------------

   procedure Size_Request
     (Widget      : access RailMap_Widget_Record'Class;
      Requisition : in Gtk_Requisition_Access);

   procedure Size_Request
     (Widget      : access RailMap_Widget_Record'Class;
      Requisition : in Gtk_Requisition_Access)
   --  This function is called by gtk+ when the widget is realized.
   --  It should modify Requisition to ask for a appropriate size
   --  for the widget. Note that the widget will not necessary have
   --  that size, it could be bigger, depending on what it is
   --  contained into. See the signal "size_allocate" too. More
   --  information on the size requisition process can be found in
   --  the book "Gtk+/Gnome Application Development" by Havoc
   --  Pennington.
   is
   begin
      Requisition.Width := Widget.MapWidth;
      Requisition.Height := Widget.MapHeight;

      --  Stop the signal from being propagated to the parent's default
      --  size_request function
      Gtk.Handlers.Emit_Stop_By_Name (Widget, "size_request");
   end Size_Request;

   -------------------
   -- Size_Allocate --
   -------------------

   procedure Size_Allocate
     (Widget     : access RailMap_Widget_Record'Class;
      Allocation : in     Gtk_Allocation_Access);

   procedure Size_Allocate
     (Widget     : access RailMap_Widget_Record'Class;
      Allocation : in     Gtk_Allocation_Access)
   --  This function is called once gtk has decided what size and
   --  position the widget will actually have, or everytime the
   --  widget is resized. This would be a good time for instance
   --  for resizing the component sub-widgets. Note that we have to
   --  move and resize the widget ourselves, and that for such a
   --  simple case, we could simply rely on the ancestor's
   --  size_allocation function.
   is
   begin
      if Realized_Is_Set (Widget) then
         Gdk.Window.Move_Resize (Get_Window (Widget),
                                 Allocation.X, Allocation.Y,
                                 Gint (Allocation.Width),
                                 Gint (Allocation.Height));
      end if;
      Gtk.Handlers.Emit_Stop_By_Name (Widget, "size_allocate");
   end Size_Allocate;


   -------------
   -- Gtk_New --
   -------------

   procedure Gtk_New (Widget : out RailMap_Widget) is
      --  Used to create a new widget
   begin
      Widget := new RailMap_Widget_Record;
      Initialize (Widget);
   end Gtk_New;

   ----------------
   -- Initialize --
   ----------------

   procedure Initialize (Widget : access RailMap_Widget_Record'Class) is
   begin
      --  We need to call the ancestor's Initialize function to create
      --  the underlying C object.
      Gtk.Drawing_Area.Initialize (Widget);

      --  The following call is required to initialize the class record,
      --  and the new signals created for this widget.
      --  Note that for now you can only create basic signals (whose
      --  callbacks do not have any parameters), and that you just have
      --  to put their name in a table.
      --  Note also that we keep Class_Record, so that the memory allocation
      --  is done only once.
      --        Gtk.Object.Initialize_Class_Record
      --          (Widget, Signals, Class_Record, "TestGtkRailMapWidget");

      --  Note: We can not create the GC here, since the widget is not
      --  realized yet, and thus has no window available. This could be
      --  done be using the "realize" signal, if we really want to be clean,
      --  here we just do it in the redraw function (a little bit slower).

      --        Widget.Radius := 60;
      Widget.MapWidth  := 350;
      Widget.MapHeight := 320;

      --  We want to get Button_Release events
      Set_Events (Widget, Exposure_Mask or Button_Release_Mask
                  or Button_Press_Mask);

      --  Set up the appropriate callbacks to redraw, ...
      Return_Boolean_Cb.Connect
        (Widget, "expose_event",
         Return_Boolean_Cb.To_Marshaller (Draw_RailMap'Access), True);

      Size_Cb.Connect
        (Widget, "size_request",
         Requisition_Marshaller.To_Marshaller (Size_Request'Access));

      Allocation_Cb.Connect
        (Widget, "size_allocate",
         Allocation_Marshaller.To_Marshaller (Size_Allocate'Access));
   end Initialize;

   procedure SetTrainPosition (Widget : access RailMap_Widget_Record'Class;
                               Train : Range_Trains;
                               new_from : Station_T;
                               new_to   : Station_T;
                               new_pos  : Integer;
                               new_over : Integer
                              ) is
   begin
      Widget.AllTrainsPositions(Train)
        :=(stat_from=>new_from, stat_to=>new_to, pos=>new_pos, over=>new_over);
      if Widget.MaxTrainUsed<Train then
         Widget.MaxTrainUsed := Train;
      end if;

      Queue_Draw(Widget);
   end SetTrainPosition;

end Net_View;
