--------------------------------------------------------
--  Paolo Santi - 2007                                --
--  Railway (il ritorno) 1.1                          --
--  Universita' di Padova                             --
--  Anno accademico 2006/2007                         --
--  Laurea Magistrale in Informatica                  --
--  Corso di Sistemi Concorrenti e Distribuiti        --
--                                                    --
--  Rielaborazione di un progetto di Alessandro Temil --
--                                                    --
--------------------------------------------------------
--  Copyright (C) 2006                                --
--  Alessandro Temil                                  --
--  Railway 1.0                                       --
--  Concurrent and Distributed Systems class project  --
--  Master Degree in Computer Science                 --
--  Academic year 2005/6                              --
--  Dept. of Pure and Applied Mathematics             --
--  University of Padua, Italy                        --
--                                                    --
--  This program is free software; you can            --
--  redistribute it and/or modify it under the terms  --
--  of the GNU General Public License as published    --
--  by the Free Software Foundation; either           --
--  version 2 of the License, or (at your option)     --
--  any later version.                                --
--------------------------------------------------------

with Railway.Types;          use Railway.Types;
with Railway.Platform;       use Railway.Platform;
with Railway.Ticket_Counter; use Railway.Ticket_Counter;
with Ada.Strings.Unbounded;  use Ada.Strings.Unbounded;

package Railway.Network is

   type Station_Map_T is record
      Num : Station_T;
      Name : Unbounded_String;
   end record;

   --  Station Names
   NULLSTNAME : aliased String := ("Null Station Name");
   PARISSTNAME : aliased String := ("Paris Gare du Nord");
   BRUXELLESSTNAME : aliased String := ("Bruxelles Midi");
   WIENSTNAME : aliased String := ("Wien Hauptbahnhof");
   MADRIDSTNAME : aliased String := ("Madrid Atocha");
   VENEZIASTNAME : aliased String := ("Venezia Santa Lucia");

   type Stations_T is array (Integer range <>) of Station_Map_T;

   Stations_Map : Stations_T :=
     ((Venezia, To_Unbounded_String (VENEZIASTNAME)),
      (Madrid, To_Unbounded_String (MADRIDSTNAME)),
      (Paris, To_Unbounded_String (PARISSTNAME)),
      (Wien, To_Unbounded_String (WIENSTNAME)),
      (Bruxelles, To_Unbounded_String (BRUXELLESSTNAME)));

   --  Ticket counter Names
   Ticket_Counter : constant String := "ticket counter";
   TC_VENEZIA   : aliased String := "Venezia " & Ticket_Counter;
   TC_MADRID    : aliased String := "Madrid " & Ticket_Counter;
   TC_PARIS     : aliased String := "Paris " & Ticket_Counter;
   TC_BRUXELLES : aliased String := "Bruxelles " & Ticket_Counter;
   TC_WIEN      : aliased String := "Wien " & Ticket_Counter;
   TC_NESSUNA   : aliased String := "Nessuna stazione " & Ticket_Counter;

   --  Ticket counters
   Ticket_Counter_Venezia   : aliased Ticket_Counter_T (TC_VENEZIA'Access);
   Ticket_Counter_Madrid    : aliased Ticket_Counter_T (TC_MADRID'Access);
   Ticket_Counter_Paris     : aliased Ticket_Counter_T (TC_PARIS'Access);
   Ticket_Counter_Bruxelles : aliased Ticket_Counter_T (TC_BRUXELLES'Access);
   Ticket_Counter_Wien      : aliased Ticket_Counter_T (TC_WIEN'Access);
   Ticket_Counter_Nessuna   : aliased Ticket_Counter_T (TC_NESSUNA'Access);

   All_Ticket_Counters : array(Station_T) of Ticket_Counter_Ref_T :=
     (
      Venezia => Ticket_Counter_Venezia'Access,
      Madrid => Ticket_Counter_Madrid'Access,
      Paris => Ticket_Counter_Paris'Access,
      Bruxelles => Ticket_Counter_Bruxelles'Access,
      Wien => Ticket_Counter_Wien'Access,
      NullStation => Ticket_Counter_Nessuna'Access
     );

   type Train_Def_T is
      record
         Route : Path_Ref_T;
         Seats : Positive;
         prior : Positive;
         Name : String_Ref_T;
         category : Train_Type_T;
      end record;

   --  Train 1
   --  EC_MAVE - route
   EC_MAVE_Path : aliased Train_Path_T :=
     (
      1 => (Pl_Madrid_1_Platform_ID, Madrid_Paris_Track_ID, 1),
      2 => (Pl_Paris_2_Platform_ID, Paris_Venezia_Track_ID, 2),
      3 => (Pl_Venezia_1_Platform_ID, Venezia_Paris_Track_ID, 3),
      4 => (Pl_Paris_1_Platform_ID, Paris_Madrid_Track_ID, 4)
     );
   --  EC_MAVE - description
   Madrid_Venezia_EC : aliased String := "[Madrid-Venezia EC]";

   --  Train 2
   --  EC Wien - Madrid
   --  EC_VIMA - route
   EC_VIMA_Path : aliased Train_Path_T :=
     (
      1 => (Pl_Wien_1_Platform_ID, Wien_Bruxelles_Track_ID, 1),
      2 => (Pl_Bruxelles_2_Platform_ID, Bruxelles_Paris_Track_ID, 2),
      3 => (Pl_Paris_1_Platform_ID, Paris_Madrid_Track_ID, 3),
      4 => (Pl_Madrid_1_Platform_ID, Madrid_Paris_Track_ID, 4),
      5 => (Pl_Paris_1_Platform_ID, Paris_Bruxelles_Track_ID, 5),
      6 => (Pl_Bruxelles_1_Platform_ID, Bruxelles_Wien_Track_ID, 6)
     );
   --  EC_VIMA - description
   Vienna_Madrid_EC : aliased String := "[Wien-Madrid EC]";

   --  Train 3
   --  Regular Train Wien-Paris
   --  RT_VIPA - route
   RT_VIPA_Path : aliased Train_Path_T :=
     (
      1 => (Pl_Wien_2_Platform_ID, Wien_Bruxelles_Track_ID, 1),
      2 => (Pl_Bruxelles_2_Platform_ID, Bruxelles_Paris_Track_ID, 2),
      3 => (Pl_Paris_2_Platform_ID, Paris_Bruxelles_Track_ID, 3),
      4 => (Pl_Bruxelles_1_Platform_ID, Bruxelles_Wien_Track_ID, 4)
     );
   --  EC_VIMA - description
   Vienna_Paris_RT : aliased String := "[Wien-Paris RT]";

   --  Train 4
   --  EuroCity Bruxelles-Madrid
   --  EC_BRMA - route
   EC_BRMA_Path : aliased Train_Path_T :=
     (
      1 => (Pl_Bruxelles_1_Platform_ID, Bruxelles_Paris_Track_ID, 1),
      2 => (Pl_Paris_1_Platform_ID, Paris_Madrid_Track_ID, 2),
      3 => (Pl_Madrid_1_Platform_ID, Madrid_Paris_Track_ID, 3),
      4 => (Pl_Paris_1_Platform_ID, Paris_Bruxelles_Track_ID, 4)
     );
   --  EC_BRMA - description
   Bruxelles_Madrid_EC : aliased String := "[Bruxelles-Madrid EC]";

   --  Train 5
   --  RT Bruxelles-Madrid
   --  RT_BRMA - route
   RT_BRMA_Path : aliased Train_Path_T :=
     (
      1 => (Pl_Bruxelles_1_Platform_ID, Bruxelles_Paris_Track_ID, 1),
      2 => (Pl_Paris_1_Platform_ID, Paris_Madrid_Track_ID, 2),
      3 => (Pl_Madrid_1_Platform_ID, Madrid_Paris_Track_ID, 3),
      4 => (Pl_Paris_1_Platform_ID, Paris_Bruxelles_Track_ID, 4)
     );

   --  RT_BRMA - description
   Bruxelles_Madrid_RT : aliased String := "[Bruxelles-Madrid RT]";

   All_Trains_Def :  array(1..5) of aliased Train_Def_T :=
     (
      1 => (
            Route =>EC_MAVE_Path'Access,
            Seats => 10, --  DEBUG
            prior => Priority_EC,
            Name => Madrid_Venezia_EC'Access,
            category => EC),
      2 => (
            Route =>EC_VIMA_Path'Access,
            Seats => 10,
            prior => Priority_EC,
            Name => Vienna_Madrid_EC'Access,
            category => EC),
      3 => (
            Route =>RT_VIPA_Path'Access,
            Seats => 15,
            prior => Priority_IR,
            Name => Vienna_Paris_RT'Access,
            category => IR),
      4 => (
            Route =>EC_BRMA_Path'Access,
            Seats => 8,
            prior => Priority_EC,
            Name => Bruxelles_Madrid_EC'Access,
            category => EC),
      5 => (
            Route =>RT_BRMA_Path'Access,
            Seats => 25,
            prior => Priority_IR,
            Name => Bruxelles_Madrid_RT'Access,
            category => IR)
     );

   --  EC_MAVE - Ticket_DB
   Ticket_DB_ECMAVE : aliased Ticket_DB_T
     (Train_Name => All_Trains_Def(Madrid_Venezia_EC_ID).Name,
      Train_Number => Madrid_Venezia_EC_ID,
      Route => All_Trains_Def(Madrid_Venezia_EC_ID).Route,
      Routefirst => All_Trains_Def(Madrid_Venezia_EC_ID).Route'First,
      Routelast => All_Trains_Def(Madrid_Venezia_EC_ID).Route'Last,
      NumberofSeats => All_Trains_Def(Madrid_Venezia_EC_ID).Seats);

      --  EC_VIMA - Ticket_DB
   Ticket_DB_ECVIMA : aliased Ticket_DB_T
     (Train_Name => All_Trains_Def(Wien_Madrid_EC_ID).Name,
      Train_Number => Wien_Madrid_EC_ID,
      Route => All_Trains_Def(Wien_Madrid_EC_ID).Route,
      Routefirst => All_Trains_Def(Wien_Madrid_EC_ID).Route'First,
      Routelast => All_Trains_Def(Wien_Madrid_EC_ID).Route'Last,
      NumberofSeats => All_Trains_Def(Wien_Madrid_EC_ID).Seats);

   --  EC_BRMA - Ticket_DB
   Ticket_DB_ECBRMA : aliased Ticket_DB_T
     (Train_Name => All_Trains_Def(Bruxelles_Madrid_EC_ID).Name,
      Train_Number => Bruxelles_Madrid_EC_ID,
      Route => All_Trains_Def(Bruxelles_Madrid_EC_ID).Route,
      Routefirst => All_Trains_Def(Bruxelles_Madrid_EC_ID).Route'First,
      Routelast => All_Trains_Def(Bruxelles_Madrid_EC_ID).Route'Last,
      NumberofSeats => All_Trains_Def(Bruxelles_Madrid_EC_ID).Seats);

   All_Ticket_DB : array(1..5) of Ticket_DB_Ref_T :=
     (
      Ticket_DB_ECMAVE'Access,
      Ticket_DB_ECVIMA'Access,
      null,
      Ticket_DB_ECBRMA'Access,
      null
     );

end Railway.Network;
