--------------------------------------------------------
--  Paolo Santi - 2007                                --
--  Railway (il ritorno) 1.1                          --
--  Universita' di Padova                             --
--  Anno accademico 2006/2007                         --
--  Laurea Magistrale in Informatica                  --
--  Corso di Sistemi Concorrenti e Distribuiti        --
--                                                    --
--  Rielaborazione di un progetto di Alessandro Temil --
--                                                    --
--------------------------------------------------------
--  Copyright (C) 2006                                --
--  Alessandro Temil                                  --
--  Railway 1.0                                       --
--  Concurrent and Distributed Systems class project  --
--  Master Degree in Computer Science                 --
--  Academic year 2005/6                              --
--  Dept. of Pure and Applied Mathematics             --
--  University of Padua, Italy                        --
--                                                    --
--  This program is free software; you can            --
--  redistribute it and/or modify it under the terms  --
--  of the GNU General Public License as published    --
--  by the Free Software Foundation; either           --
--  version 2 of the License, or (at your option)     --
--  any later version.                                --
--------------------------------------------------------

with Distrailway.Central_Control;
with Distrailway.Notice_Boards;   use Distrailway.Notice_Boards;
with Railway.Timeref;   use Railway.Timeref;

--  with Ada.Text_IO;           use Ada.Text_IO;

package body Railway.Platform is

   --  ***********************************
   --
   --  Platform_T
   --
   --  ***********************************
   protected body Platform_T is

      entry Train_Arrived
        (Train_Name : String;
         Train_Number : Range_Trains;
         Free_Seats : Integer;
         Seats : Integer;
         Destination : String) when True is
      begin

         Passed := Clock - Start_Time;

         Distrailway.Central_Control.Upd_Train
           (Integer (To_Duration (Passed)),
            Train_Number,
            Train_Name,
            Seats - Free_Seats,
            Seats,
            "queued for platform " & Name.all & " " & Station.all);

         Distrailway.Notice_Boards.Update
           (Integer (To_Duration (Passed)),
            Station.all,
            Name.all,
            Train_Name & " to " & Destination
            & " queued for access to platform");

         requeue Stopping;
      end Train_Arrived;

      entry Stopping
        (Train_Name : String;
         Train_Number : Range_Trains;
         Free_Seats : Integer;
         Seats : Integer;
         Destination : String) when Stopped_Train = null is
      begin

         Stopped_Train := new String'(Train_Name);
         Stopped_Train_Number := Train_Number;

         Passed := Clock - Start_Time;

         Distrailway.Central_Control.Upd_Train
           (Integer (To_Duration (Passed)),
            Train_Number,
            Train_Name,
            Seats - Free_Seats,
            Seats,
            "arriving at platform " & Name.all & " in " & Station.all);

         Distrailway.Notice_Boards.Update
           (Integer (To_Duration (Passed)),
            Station.all,
            Name.all,
            Train_Name & " to " & Destination
            & " arrived at the platform");

         tFree_Seats := Free_Seats;
         tSeats := Seats;

      end Stopping;

      procedure Boarding
        (Train_Name : String;
         Next_Stage : Integer;
         Destination : String) is
      begin

         TrainBoarding := new String'(Train_Name);
         Train_Next_Stage := Next_Stage;
         Passed := Clock - Start_Time;

         Distrailway.Central_Control.Upd_Train
           (Integer (To_Duration (Passed)),
            Stopped_Train_Number,
            Train_Name,
            tSeats - tFree_Seats,
            tSeats,
            "boarding passengers at platform "
            & Name.all & " in " & Station.all);

         Distrailway.Notice_Boards.Update
           (Integer (To_Duration (Passed)),
            Station.all,
            Name.all,
            Train_Name & " to " & Destination
            & " boarding passengers");

      end Boarding;

      procedure Close_Doors
        (Train_Name : String;
         Free_Seats : out Integer;
         Destination : String) is
      begin

         Stopped_Train := null;
         TrainBoarding := null;

         Free_Seats := tFree_Seats;

         Passed := Clock - Start_Time;

         Distrailway.Central_Control.Upd_Train
           (Integer (To_Duration (Passed)),
            Stopped_Train_Number,
            Train_Name,
            tSeats - tFree_Seats,
            tSeats,
            "leaving platform " & Name.all & " " & Station.all);

         Distrailway.Notice_Boards.Update
           (Integer (To_Duration (Passed)),
            Station.all,
            Name.all,
            Train_Name & " to " & Destination & " departing");
      end Close_Doors;





      entry Depart(for T in Range_Trains)
        (Passenger_Route : Travel_Ext_Route_T;
         Travel_Index : in out Integer;
         Passenger_Name : String)
        when TrainBoarding = null is
      begin
         requeue GetOn(T);
      end Depart;



      entry GetOn(for T in Range_Trains)
        (Passenger_Route : Travel_Ext_Route_T;
         Travel_Index : in out Integer;
         Passenger_Name : String)
        when TrainBoarding /= null
        and tFree_Seats > 0 is

         Index_Last_Segment_current_leg : Integer;
         Next_Segment : Integer;
      begin

         if Train_Next_Stage/=Passenger_Route(Travel_Index).Stg_From then
            --              Put_Line ("Depart requeue Depart = " & Passenger_Name
            --                        & "  Travel_Index  = " & Integer'Image(Travel_Index)
            --                        & "  Passenger_Next_Stage  = " & Integer'Image(Passenger_Route(Travel_Index).Stg_From)
            --                        & "  Train_Next_Stage  = " & Integer'Image(Train_Next_Stage)
            --                       );
            requeue Depart(T);
         end if;

         Index_Last_Segment_current_leg := Passenger_Route'Last;
         Next_Segment := Travel_Index;

         tFree_Seats := tFree_Seats - 1;

         Passed := Clock - Start_Time;

         Distrailway.Central_Control.Upd_Train
           (Integer (To_Duration (Passed)),
            Stopped_Train_Number,
            Stopped_Train.all,
            tSeats - tFree_Seats,
            tSeats,
            "boarding passengers at platform "
            & Name.all & " in " & Station.all);

         Distrailway.Central_Control.Upd_Passenger
           (Integer (To_Duration (Passed)),
            Passenger_Name,
            "boards " & TrainBoarding.all & " at platform "
            & Name.all & " in " & Station.all);

         if(Travel_Index = Passenger_Route'Last) then
            requeue AllPlatformsReference
              (Passenger_Route(Travel_Index).Plat_To).
              Arrive (Stopped_Train_Number);
         else
            requeue AllPlatformsReference
              (Passenger_Route(Travel_Index).Plat_To).Change_Train (Stopped_Train_Number);
         end if;
      end GetOn;


      entry Arrive(for T in Range_Trains)
        (Passenger_Route : Travel_Ext_Route_T;
         Travel_Index : in out Integer;
         Passenger_Name : String)
      when Stopped_Train /= null and Stopped_Train_Number = T is
         Current_Travel_Index : Integer := Travel_Index;

      begin
         tFree_Seats := tFree_Seats + 1;

         Passed := Clock - Start_Time;

         Distrailway.Central_Control.Upd_Passenger
           (Integer (To_Duration (Passed)),
            Passenger_Name,
            "alights train " & Stopped_Train.all & " at platform "
            & Name.all & " in " & Station.all);

         Passed := Clock - Start_Time;

         Distrailway.Central_Control.Upd_Train
           (Integer (To_Duration (Passed)),
            Stopped_Train_Number,
            Stopped_Train.all,
            tSeats - tFree_Seats,
            tSeats,
            "arriving at platform " & Name.all & " in " & Station.all);

         Current_Travel_Index := Current_Travel_Index + 1;

         if Passenger_Route(Travel_Index).Train_Type = EC then
            All_The_Ticket_DB_Refs
              (Passenger_Route(Travel_Index).Train).Release_Ticket
              (Passenger_Route, Travel_Index, Passenger_Name);
         end if;
      end Arrive;



      entry Change_Train(for T in Range_Trains)
        (Passenger_Route : Travel_Ext_Route_T;
         Travel_Index : in out Integer;
         Passenger_Name : String)
      when Stopped_Train /= null
        and Stopped_Train_Number = T is

      begin
         tFree_Seats := tFree_Seats + 1;

         Passed := Clock - Start_Time;

         Distrailway.Central_Control.Upd_Passenger
           (Integer (To_Duration (Passed)),
            Passenger_Name,
            "changes over at platform " & Name.all & " in " & Station.all);

         Passed := Clock - Start_Time;

         Distrailway.Central_Control.Upd_Train
           (Integer (To_Duration (Passed)),
            Stopped_Train_Number,
            Stopped_Train.all,
            tSeats - tFree_Seats,
            tSeats,
            "arriving at platform " & Name.all & " " & Station.all);

         if Passenger_Route(Travel_Index).Train_Type = EC then
            All_The_Ticket_DB_Refs
              (Passenger_Route(Travel_Index).Train).Release_Ticket
              (Passenger_Route, Travel_Index, Passenger_Name);
         end if;
         Travel_Index := Travel_Index + 1;

         requeue AllPlatformsReference
           (Passenger_Route(Travel_Index).Plat_From).
           Depart(Passenger_Route(Travel_Index).Train);

      end Change_Train;

   end Platform_T;


   --  ***********************************
   --
   --  Ticket_DB_T
   --
   --  ***********************************
   protected body Ticket_DB_T is

      entry Book_Leg
        (Passenger_Route : Travel_Ext_Route_T;
         Travel_Index : in out Integer;
         Passenger_Name : String) when True is

         Index_First_stage : constant Integer := Passenger_Route (Travel_Index).Stg_From;
      begin


         --           Put_Line ("Book_Leg = " & Passenger_Name
         --                     & "  Travel_Index  = " & Integer'Image(Travel_Index)
         --                     & "  First_stage  = " & Integer'Image(Index_First_stage)
         --                    );

         requeue Book_Ticket (Index_First_stage);

      end Book_Leg;



      entry Book_Ticket (for N_Stage in Routefirst..Routelast)
        (Passenger_Route : Travel_Ext_Route_T;
         Travel_Index : in out Integer;
         Passenger_Name : String)
      when Available_Tickets(N_Stage) > 0 is

         Index_Next_stage : Integer := N_Stage+1;
         R_Low : Integer;
         R_High : Integer;

         More_Segment : Boolean := False;
         Index_First_Segment_leg : Integer;
         book_more_leg : Boolean := False;
         Index_Train_with_lower_Number : Integer := 0;
         ix : Integer;

      begin
         ix := N_Stage;
         Available_Tickets (ix) := Available_Tickets (ix) - 1;

         --  test next stage overflow then wrap index to start
         if Index_Next_stage>Passenger_Route (Travel_Index).Stg_Tot then
            Index_Next_stage := 1;
         end if;

         --           Put_Line (
         --                     Train_Name.all
         --                     & " Book_Ticket = " & Passenger_Name
         --                     & "  Travel_Index  = " & Integer'Image(Travel_Index)
         --                     & "  N_Stage  = " & Integer'Image(N_Stage)
         --                     & "  Index_Next_stage  = " & Integer'Image(Index_Next_stage)
         --                     & "  ix  = " & Integer'Image(ix)
         --                     & "  Available_Tickets (ix)  = " & Integer'Image(Available_Tickets (ix))
         --                    );

         --  looks for next stage
         R_Low  := Passenger_Route (Travel_Index).Stg_From;
         R_High := Passenger_Route (Travel_Index).Stg_To;
         if(Index_Next_stage/=R_High) then
            --  yes, there is a new stage
            More_Segment := True;
         end if;

         if More_Segment then
            requeue Book_Ticket (Index_Next_stage);
         end if;

         --  if no more stages, next leg
         Index_Train_with_lower_Number := Travel_Index;

         --  test if there is some other leg to book-on
         --  search the minimum train ID greater than actual
         for Legso_ix in Passenger_Route'First .. Passenger_Route'Last loop

            --  the new train to book must be EC type
            if (Passenger_Route(Legso_ix).Train_Type = EC) then

               --  the first new booking
               if not book_more_leg
                 and Passenger_Route (Legso_ix).Train >
                 Passenger_Route (Travel_Index).Train
               then
                  book_more_leg := True;
                  Index_Train_with_lower_Number := Legso_ix;
               end if;

               --  the others new booking
               if book_more_leg
                 and Passenger_Route (Legso_ix).Train >
                 Passenger_Route (Travel_Index).Train
                 and Passenger_Route (Legso_ix).Train
                 < Passenger_Route (Index_Train_with_lower_Number).Train
               then
                  Index_Train_with_lower_Number := Legso_ix;
               end if;

            end if;

         end loop;

         --  at least another booking needed
         if book_more_leg then
            --  requeue to the next train's ticket DB
            Travel_Index := Index_Train_with_lower_Number;
            requeue All_The_Ticket_DB_Refs(Passenger_Route (Travel_Index).Train).Book_Leg;
         end if;

         --  when no more Legs to book, Depart
         Index_First_Segment_leg := Passenger_Route'First;
         Travel_Index := Passenger_Route'First;

         Passed := Clock - Start_Time;

         Distrailway.Central_Control.Upd_Passenger
           (Integer (To_Duration (Passed)),
            Passenger_Name,
            "is waiting for train "
            & Integer'Image(Passenger_Route(Passenger_Route'First).Train)
            & " at platform "
            & Platforms_Names(Passenger_Route(Passenger_Route'First).Plat_From));

         requeue AllPlatformsReference
           (Passenger_Route(Travel_Index).Plat_From).
           Depart(Passenger_Route(Travel_Index).Train);

      end Book_Ticket;



      procedure Release_Ticket
        (Passenger_Route : Travel_Ext_Route_T;
         Travel_Index : in out Integer;
         Passenger_Name : String) is

         Index_Stop  : constant Integer:= Passenger_Route (Travel_Index).Stg_To;
         Index_Stage : Integer := Passenger_Route (Travel_Index).Stg_From;
      begin

         --  release ticket for the train before
         while Index_Stage /= Index_Stop loop

            Available_Tickets (Index_Stage) :=
              Available_Tickets (Index_Stage) + 1;

            --              Put_Line (
            --                        Train_Name.all
            --                        & " Release_Ticket = " & Passenger_Name
            --                        & "  Travel_Index  = " & Integer'Image(Travel_Index)
            --                        & "  Stage  = " & Integer'Image(Index_Stage)
            --                        & "  Available_Tickets (Index_Stage)  = " & Integer'Image(Available_Tickets (Index_Stage))
            --                       );

            Index_Stage := Index_Stage + 1;
            if Index_Stage>Passenger_Route (Travel_Index).Stg_Tot then
               Index_Stage := 1;
            end if;

         end loop;

      end Release_Ticket;

   end Ticket_DB_T;



end Railway.Platform;
