--------------------------------------------------------
--  Paolo Santi - 2007                                --
--  Railway (il ritorno) 1.1                          --
--  Universita' di Padova                             --
--  Anno accademico 2006/2007                         --
--  Laurea Magistrale in Informatica                  --
--  Corso di Sistemi Concorrenti e Distribuiti        --
--                                                    --
--  Rielaborazione di un progetto di Alessandro Temil --
--                                                    --
--------------------------------------------------------
--  Copyright (C) 2006                                --
--  Alessandro Temil                                  --
--  Railway 1.0                                       --
--  Concurrent and Distributed Systems class project  --
--  Master Degree in Computer Science                 --
--  Academic year 2005/6                              --
--  Dept. of Pure and Applied Mathematics             --
--  University of Padua, Italy                        --
--                                                    --
--  This program is free software; you can            --
--  redistribute it and/or modify it under the terms  --
--  of the GNU General Public License as published    --
--  by the Free Software Foundation; either           --
--  version 2 of the License, or (at your option)     --
--  any later version.                                --
--------------------------------------------------------

with Railway.Platform; use Railway.Platform;
with Railway.Tracks; use Railway.Tracks;
with Ada.Real_Time; use Ada.Real_Time;
with Railway.Stations; use Railway.Stations;
with Railway.Timeref; use Railway.Timeref;
with Distrailway.Central_Control;
with Distrailway.Net_Map;


package body Railway.Train is

   --  Time that the Trains remain Stopped
   Stop_Time : constant Time_Span := Milliseconds(3_000);

   task body Train_T is

      Current_Stage : Train_Stage_T;
      Arrival_Platform : Platform_ID_T;

      Index_Next_Stage : Integer;
      Next_Stage : Train_Stage_T;

      Traversing_Time : Time_Span;
      Next_Arrival_Platform : Platform_ID_T;

      Capacity : Integer := Seats;

   begin
      loop
         for Stage_Index in Route'Range loop
            Current_Stage := Route (Stage_Index);
            Arrival_Platform := Current_Stage.Departure_Platform;

            if Stage_Index = Route'Last then
               Index_Next_Stage := Route'First;
            else
               Index_Next_Stage := Stage_Index + 1;
            end if;

            Next_Stage := Route (Index_Next_Stage);
            Next_Arrival_Platform := Next_Stage.Departure_Platform;

            --  Put_Line ("Arrival Platform = " & Integer'Image(Arrival_Platform));
            declare
               Arrival_Platform_Name : constant String := Get_Platform_Station (Arrival_Platform); --  "Prossima fermata...";
               Next_Destination_Name : constant String := Get_Platform_Station (Next_Arrival_Platform); --  "Prossima fermata...";
               Train_Name : constant String := Name.all;
            begin
               --  Put_Line ("Il treno  " & Train_Name & "  sta andando a  " & Next_Destination);
               --  Put_Line ("Before Capacity = " & Integer'Image(Capacity));
               --  arrives at the station and waits for the platform
               Train_Arrived_Platform
                 (Arrival_Platform,
                  Train_Name, Number,
                  Capacity, Seats, Arrival_Platform_Name);

               Distrailway.Net_Map.Update_Train_Position
                 (Integer (To_Duration (Passed)),
                  Number,
                  Name.all,
                  Seats-Capacity,
                  Seats,
                  " arrived to " & Next_Destination_Name,
                  Platforms_Station_Map(Arrival_Platform),
                  Platforms_Station_Map(Arrival_Platform),
                  0,
                  100
                 );

               delay To_Duration (Stop_Time);

               --  boards passengers
               Boarding_Platform
                 (Arrival_Platform,
                  Train_Name,
                  Stage_Index,
                  Next_Destination_Name);

               delay To_Duration (Stop_Time);

               --  departs
               Close_Doors_Platform
                 (Arrival_Platform,
                  Train_Name,
                  Capacity,
                  Next_Destination_Name);

               delay To_Duration (Stop_Time);


               --  calc traversing time
               Traversing_Time :=
                 Milliseconds (Get_Traversing_Time(Current_Stage.Segment_Route));

               --  seizes the track
               Enter_Track
                 (Current_Stage.Segment_Route, Number, Name.all, Capacity, Seats);

               --  go to destination
               declare
                  All_Length : constant Integer :=
                    Get_Track_Length(Current_Stage.Segment_Route);
                  Done_Length : Integer := 0;
                  Step_Length : constant Integer := 100;
                  Step_Time   : Integer;
                  Passed      : Time_Span;

               begin
                  while Done_Length<All_Length loop
                     if All_Length-Done_Length>=Step_Length then
                        Step_Time   := Step_Length;
                        Done_Length := Done_Length + Step_Length;
                     else
                        Step_Time   := All_Length-Done_Length;
                        Done_Length := All_Length;
                     end if;
                     delay To_Duration (Milliseconds(Step_Time*Scaling_Factor));
                     Passed := Clock - Start_Time;

                     --  update position in map
                     Distrailway.Net_Map.Update_Train_Position
                       (Integer (To_Duration (Passed)),
                        Number,
                        Name.all,
                        Seats-Capacity,
                        Seats,
                        "covered " & Integer'Image(Done_Length)
                        & "/" & Integer'Image(All_Length)
                        & "Km to " & Next_Destination_Name,
                        Platforms_Station_Map(Arrival_Platform),
                        Platforms_Station_Map(Next_Arrival_Platform),
                        Done_Length,
                        All_Length
                       );

                     Distrailway.Central_Control.Upd_Train
                       (Integer (To_Duration (Passed)),
                        Number,
                        Name.all,
                        Seats-Capacity,
                        Seats,
                        "covered " & Integer'Image(Done_Length)
                        & "/" & Integer'Image(All_Length)
                        & " to " & Next_Destination_Name);
                  end loop;
               end;

            end;

            --  releases the track
            Leave_Track(Current_Stage.Segment_Route, Name.all);
         end loop;

      end loop;

   end Train_T;

end Railway.Train;
