--------------------------------------------------------
--  Paolo Santi - 2007                                --
--  Railway (il ritorno) 1.1                          --
--  Universita' di Padova                             --
--  Anno accademico 2006/2007                         --
--  Laurea Magistrale in Informatica                  --
--  Corso di Sistemi Concorrenti e Distribuiti        --
--                                                    --
--  Rielaborazione di un progetto di Alessandro Temil --
--                                                    --
--------------------------------------------------------
--  Copyright (C) 2006                                --
--  Alessandro Temil                                  --
--  Railway 1.0                                       --
--  Concurrent and Distributed Systems class project  --
--  Master Degree in Computer Science                 --
--  Academic year 2005/6                              --
--  Dept. of Pure and Applied Mathematics             --
--  University of Padua, Italy                        --
--                                                    --
--  This program is free software; you can            --
--  redistribute it and/or modify it under the terms  --
--  of the GNU General Public License as published    --
--  by the Free Software Foundation; either           --
--  version 2 of the License, or (at your option)     --
--  any later version.                                --
--------------------------------------------------------

with System;

package Railway is
   pragma Pure;

   --  priority definitions for train tasks
   Priority_IR : constant System.Priority := System.Priority'Last - 2;
   Priority_EC : constant System.Priority := System.Priority'Last - 1;

   --  define train types
   type Train_Type_T is (IR, EC);

   --  minimum train index
   Min_Train : constant Integer := 1;
   --  maximum train index
   Max_Train : constant Integer := 100;

   --  range associated with minimum and maximum train index
   subtype Range_Trains is Integer range Min_Train .. Max_Train;

   Madrid_Venezia_EC_ID   : constant Integer := 1;
   Wien_Madrid_EC_ID      : constant Integer := 2;
   Wien_Paris_RT_ID       : constant Integer := 3;
   Bruxelles_Madrid_EC_ID : constant Integer := 4;
   Bruxelles_Madrid_RT_ID : constant Integer := 5;

   --  scale factor for calculation of the
   --  traversing time given line length
   --  time in seconds to cover 1_000 Km (!)
   Scaling_Factor : constant Positive := 5;


   --  Segments IDS
   --  all system-known tracks
   type Track_ID_T is (
                       Null_Track_ID,

                       Bruxelles_Wien_Track_ID,
                       Wien_Bruxelles_Track_ID,

                       Paris_Bruxelles_Track_ID,
                       Bruxelles_Paris_Track_ID,

                       Venezia_Paris_Track_ID,
                       Paris_Venezia_Track_ID,

                       Paris_Madrid_Track_ID,
                       Madrid_Paris_Track_ID);

   type Station_T is (Paris, Bruxelles, Madrid, Venezia, Wien, NullStation);

   --  Platforms IDS
   --  all system-known Platforms
   type Platform_ID_T is (Null_Platform_ID,

                          Pl_Venezia_1_Platform_ID,
                          Pl_Venezia_2_Platform_ID,

                          Pl_Madrid_1_Platform_ID,
                          Pl_Madrid_2_Platform_ID,

                          Pl_Paris_1_Platform_ID,
                          Pl_Paris_2_Platform_ID,

                          Pl_Bruxelles_1_Platform_ID,
                          Pl_Bruxelles_2_Platform_ID,

                          Pl_Wien_1_Platform_ID,
                          Pl_Wien_2_Platform_ID);

   Platforms_Names : constant array(Platform_ID_T) of String(1..12) :=
     ("Null_Platfor",

      "Pl_Venezia_1",
      "Pl_Venezia_2",

      "Pl_Madrid_1 ",
      "Pl_Madrid_2 ",

      "Pl_Paris_1  ",
      "Pl_Paris_2  ",

      "Pl_Bruxell_1",
      "Pl_Bruxell_2",

      "Pl_Wien_1   ",
      "Pl_Wien_2   "
     );

   Platforms_Station_Map : constant array(Platform_ID_T) of Station_T :=
     (NullStation,
      Venezia,
      Venezia,
      Madrid,
      Madrid,
      Paris,
      Paris,
      Bruxelles,
      Bruxelles,
      Wien,
      Wien
      );

   --  train journey definition: stage, path
   --  a stage is a departure platform and next track associated
   type Train_Stage_T is
      record
         Departure_Platform : Platform_ID_T := Null_Platform_ID;
         Segment_Route      : Track_ID_T := Null_Track_ID;
         Stage_Number       : Integer := -1;
      end record;

   --  a (closed) train path
   --  the last stage is implicitly connected to the first
   type Train_Path_T is array (Integer range <>) of Train_Stage_T;


   --  passenger travel definition: leg
   --  a traveler's travel is a route (an array)
   --  made up of Travel_Leg_T
   type Travel_Leg_T is record
      --  train identifier (given)
      Train : Range_Trains;
      --  starting station of leg (given)
      Station_From  : Station_T;
      --  ending station of leg (given)
      Station_To    : Station_T;

   end record;

   --  a route is an arry of legs
   type Travel_Route_T is array (Integer range <>) of Travel_Leg_T;


end Railway;
